@@ -44,7 +44,7 @@ namespace GLib {
 
 		public static Opaque GetOpaque (IntPtr o, Type type, bool owned)
 		{
-			Opaque opaque = (Opaque)Activator.CreateInstance (type, new object[] { o });
+			Opaque opaque = FastActivator.CreateOpaque (o, type);
 			if (owned) {
 				if (opaque.owned) {
 					// The constructor took a Ref it shouldn't have, so undo it
@@ -87,7 +87,8 @@ namespace GLib {
 
 		~Opaque ()
 		{
-			Dispose ();
+			// for compat.  All subclasses should have
+			// generated finalizers if needed now.
 		}
 
 		public virtual void Dispose ()
@@ -113,6 +114,14 @@ namespace GLib {
 			}
 		}
 
+		public IntPtr OwnedCopy {
+			get {
+				Opaque result = Copy (Handle);
+				result.Owned = false;
+				return result.Handle;
+			}
+		}
+
 		public bool Owned {
 			get {
 				return owned;
