@@ -23,6 +23,7 @@ namespace GLib {
 
 	using System;
 	using System.Collections;
+	using System.Collections.Generic;
 	using System.Runtime.InteropServices;
 
 	internal class ClosureInvokedArgs : EventArgs {
@@ -56,31 +57,32 @@ namespace GLib {
 
 		IntPtr handle;
 		IntPtr raw_closure;
-		string name;
+		internal string name;
 		uint id = UInt32.MaxValue;
 		System.Type args_type;
-		Delegate custom_marshaler;
-		GCHandle gch;
+		GCHandle? gch;
+		internal bool before_closure;
 
-		static Hashtable closures = new Hashtable ();
+		static Dictionary<IntPtr, SignalClosure> closures = new Dictionary<IntPtr, SignalClosure> (IntPtrEqualityComparer.Instance);
 
-		public SignalClosure (IntPtr obj, string signal_name, System.Type args_type)
+		public SignalClosure (IntPtr obj, string signal_name, System.Type args_type, bool before_closure)
 		{
 			raw_closure = glibsharp_closure_new (Marshaler, Notify, IntPtr.Zero);
 			closures [raw_closure] = this;
 			handle = obj;
 			name = signal_name;
 			this.args_type = args_type;
+			this.before_closure = before_closure;
 		}
 
-		public SignalClosure (IntPtr obj, string signal_name, Delegate custom_marshaler, Signal signal)
+		public SignalClosure (IntPtr obj, string signal_name, Delegate custom_marshaler, Signal signal, bool before_closure)
 		{
 			gch = GCHandle.Alloc (signal);
 			raw_closure = g_cclosure_new (custom_marshaler, (IntPtr) gch, Notify);
 			closures [raw_closure] = this;
 			handle = obj;
 			name = signal_name;
-			this.custom_marshaler = custom_marshaler;
+			this.before_closure = before_closure;
 		}
 
 		public event EventHandler Disposed;
@@ -103,9 +105,10 @@ namespace GLib {
 		{
 			Disconnect ();
 			closures.Remove (raw_closure);
-			if (custom_marshaler != null)
-				gch.Free ();
-			custom_marshaler = null;
+			if (gch != null) {
+				gch.Value.Free ();
+				gch = null;
+			}
 			if (Disposed != null)
 				Disposed (this, EventArgs.Empty);
 			GC.SuppressFinalize (this);
@@ -121,24 +124,24 @@ namespace GLib {
 		static ClosureMarshal marshaler;
 		static ClosureMarshal Marshaler {
 			get {
-				if (marshaler == null)
-					marshaler = new ClosureMarshal (MarshalCallback);
+				if (marshaler == null) {
+					unsafe {
+						marshaler = new ClosureMarshal (MarshalCallback);
+					}
+				}
 				return marshaler;
 			}
 		}
 
 		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
-		delegate void ClosureMarshal (IntPtr closure, IntPtr return_val, uint n_param_vals, IntPtr param_values, IntPtr invocation_hint, IntPtr marshal_data);
+		unsafe delegate void ClosureMarshal (IntPtr closure, Value *return_val, uint n_param_vals, Value *param_values, IntPtr invocation_hint, IntPtr marshal_data);
 
-		static void MarshalCallback (IntPtr raw_closure, IntPtr return_val, uint n_param_vals, IntPtr param_values, IntPtr invocation_hint, IntPtr marshal_data)
+		unsafe static void MarshalCallback (IntPtr raw_closure, Value *return_val, uint n_param_vals, Value *param_values, IntPtr invocation_hint, IntPtr marshal_data)
 		{
-			string message = String.Empty;
-
+			SignalClosure closure = null;
 			try {
-				SignalClosure closure = closures [raw_closure] as SignalClosure;
-				message = "Marshaling " + closure.name + " signal";
-				Value objval = (Value) Marshal.PtrToStructure (param_values, typeof (Value));
-				GLib.Object __obj = objval.Val as GLib.Object;
+				closure = closures [raw_closure] as SignalClosure;
+				GLib.Object __obj = param_values[0].Val as GLib.Object;
 				if (__obj == null)
 					return;
 
@@ -147,29 +150,23 @@ namespace GLib {
 					return;
 				}
 
-				SignalArgs args = Activator.CreateInstance (closure.args_type, new object [0]) as SignalArgs;
+				SignalArgs args = FastActivator.CreateSignalArgs (closure.args_type);
 				args.Args = new object [n_param_vals - 1];
-				GLib.Value[] vals = new GLib.Value [n_param_vals - 1];
 				for (int i = 1; i < n_param_vals; i++) {
-					IntPtr ptr = new IntPtr (param_values.ToInt64 () + i * Marshal.SizeOf (typeof (Value)));
-					vals [i - 1] = (Value) Marshal.PtrToStructure (ptr, typeof (Value));
-					args.Args [i - 1] = vals [i - 1].Val;
+					args.Args [i - 1] = param_values [i].Val;
 				}
 				ClosureInvokedArgs ci_args = new ClosureInvokedArgs (__obj, args);
 				closure.Invoke (ci_args);
 				for (int i = 1; i < n_param_vals; i++) {
-					vals [i - 1].Update (args.Args [i - 1]);
-					IntPtr ptr = new IntPtr (param_values.ToInt64 () + i * Marshal.SizeOf (typeof (Value)));
-					Marshal.StructureToPtr (vals [i - 1], ptr, false);
+					param_values [i].Update (args.Args [i - 1]);
 				}
-				if (return_val == IntPtr.Zero || args.RetVal == null)
+				if (return_val == null || args.RetVal == null)
 					return;
 
-				Value ret = (Value) Marshal.PtrToStructure (return_val, typeof (Value));
-				ret.Val = args.RetVal;
-				Marshal.StructureToPtr (ret, return_val, false);
+				return_val->Val = args.RetVal;
 			} catch (Exception e) {
-				Console.WriteLine (message);
+				if (closure != null)
+					Console.WriteLine ("Marshaling {0} signal", closure.name);
 				ExceptionManager.RaiseUnhandledException (e, false);
 			}
 		}
@@ -179,9 +176,9 @@ namespace GLib {
 
 		static void NotifyCallback (IntPtr data, IntPtr raw_closure)
 		{
-			SignalClosure closure = closures [raw_closure] as SignalClosure;
-			if (closure != null)
-				closure.Dispose ();
+			SignalClosure closure;
+			if (closures.TryGetValue (raw_closure, out closure))
+				closure.Dispose();
 		}
 
 		static ClosureNotify notify_handler;
