@@ -23,17 +23,17 @@ namespace Pango {
 
 	public class AttrFontDesc : Attribute {
 
-		[DllImport("libpango-1.0-0.dll")]
+		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_font_desc_new (IntPtr font_desc);
 
-		[DllImport("libpango-1.0-0.dll")]
+		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_font_description_copy(IntPtr raw);
 
-		public AttrFontDesc (Pango.FontDescription font_desc) : this (pango_attr_font_desc_new (pango_font_description_copy (font_desc.Handle))) {}
+		public AttrFontDesc (Pango.FontDescription font_desc) : this (pango_attr_font_desc_new (pango_font_description_copy (font_desc.Handle)), true) {}
 
-		internal AttrFontDesc (IntPtr raw) : base (raw) {}
+		internal AttrFontDesc (IntPtr raw, bool owned) : base (raw, owned) {}
 
-		[DllImport("pangosharpglue-2")]
+		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pangosharp_attr_font_desc_get_desc (IntPtr raw);
 
 		public Pango.FontDescription Desc {
