@@ -26,9 +26,9 @@ namespace Pango {
 		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_gravity_hint_new (int hint);
 
-		public AttrGravityHint (GravityHint hint) : this (pango_attr_gravity_hint_new ((int) hint)) {}
+		public AttrGravityHint (GravityHint hint) : this (pango_attr_gravity_hint_new ((int) hint), true) {}
 
-		internal AttrGravityHint (IntPtr raw) : base (raw) {}
+		internal AttrGravityHint (IntPtr raw, bool owned) : base (raw, owned) {}
 
 		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern int pangosharp_attr_int_get_value (IntPtr raw);
