@@ -23,19 +23,19 @@ namespace Pango {
 
 	public class AttrSize : Attribute {
 
-		[DllImport("libpango-1.0-0.dll")]
+		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_size_new (int size);
 
-		[DllImport("libpango-1.0-0.dll")]
+		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_size_new_absolute (int size);
 
-		public AttrSize (int size) : this (pango_attr_size_new (size)) {}
+		public AttrSize (int size) : this (pango_attr_size_new (size), true) {}
 
-		public AttrSize (int size, bool absolute) : this (absolute ? pango_attr_size_new (size) : pango_attr_size_new_absolute (size)) {}
+		public AttrSize (int size, bool absolute) : this (absolute ? pango_attr_size_new (size) : pango_attr_size_new_absolute (size), true) {}
 
-		internal AttrSize (IntPtr raw) : base (raw) {}
+		internal AttrSize (IntPtr raw, bool owned) : base (raw, owned) {}
 
-		[DllImport("pangosharpglue-2")]
+		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern int pangosharp_attr_size_get_size (IntPtr raw);
 
 		public int Size {
@@ -44,7 +44,7 @@ namespace Pango {
 			}
 		}
 
-		[DllImport("pangosharpglue-2")]
+		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern bool pangosharp_attr_size_get_absolute (IntPtr raw);
 
 		public bool Absolute {
