@@ -26,9 +26,9 @@ namespace Pango {
 		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
 		static extern IntPtr pango_attr_variant_new (Pango.Variant variant);
 
-		public AttrVariant (Pango.Variant variant) : this (pango_attr_variant_new (variant)) {}
+		public AttrVariant (Pango.Variant variant) : this (pango_attr_variant_new (variant), true) {}
 
-		internal AttrVariant (IntPtr raw) : base (raw) {}
+		internal AttrVariant (IntPtr raw, bool owned) : base (raw, owned) {}
 
 		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
 		static extern int pangosharp_attr_int_get_value (IntPtr raw);
